 /*
  * Khoros: $Id$
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for guise
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "guise.h"

clui_info_struct *clui_info = NULL;

gui_info_struct *gui_info;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Direct manipulation GUI design tool
|
|       Purpose: main program for guise
|
|         Input:
|		int clui_info->force_flag; {TRUE if -force specified}
|
|		char *clui_info->o_file; {output UIS file}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Optional M.E. group:
|		    char *clui_info->i_file; {initial UIS file}
|		    int   clui_info->i_flag; {TRUE if -i specified}
|
|		    M.I. group:
|			char *clui_info->tb_string; {toolbox}
|			int   clui_info->tb_flag; {TRUE if -tb specified}
|
|			char *clui_info->oname_string; {object name}
|			int   clui_info->oname_flag; {TRUE if -oname specified}
|
|			char *clui_info->uis_string; {UIS file to edit (do not include path)}
|			int   clui_info->uis_flag; {TRUE if -uis specified}
|
|        Output:
|       Returns:
|
|    Written By: Danielle Argiro
|          Date: Apr 14, 1995
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv,
   char **envp)
{

	kform  *form;            /* form tree representing *.form file */
	char   *form_filestring; /* *.form file for guise    */ 
	char   *form_pathname;   /* location of guise.form   */
	char   *form_fullpath;   /* expanded guise.form path */
	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
        char    fullpath[KLENGTH];
/* -main_variable_list_end */

	khoros_initialize(argc, argv, envp, "DESIGN");
	kexit_handler(guise_free_args, NULL);

/* -main_get_args_call */
	pane = kgen_initialize(PANEPATH, KGEN_XVROUTINE, "DESIGN", "guise",
				guise_usage_additions);

	guise_get_args(pane);
	kvf_destroy_form(pane);
/* -main_get_args_call_end */

	/*
	 * Initialize xvwidgets lib
	 */
	if (!(xvw_initialize(XVW_MENUS_XVFORMS)))
	{
	     kinfo(KSTANDARD, "Could not open connection to server");
	     kexit(KEXIT_FAILURE);
	}

	/*
	 * Set location of guise.form
	 */
	if (clui_info->form_flag)
	{
	    form_filestring = kstrdup(clui_info->form_file);
	    form_pathname   = NULL;
	}
	else
	{
	    form_filestring = kstrdup("guise.form");
	    form_pathname   = kstrdup("$TOOLBOX/objects/xvroutine/guise/uis");
	}
	form_fullpath   = kfullpath(form_filestring, form_pathname, NULL);

	/*
	 * Allocate the GUI Info structure 
	 */
	gui_info = (gui_info_struct *)
				kcalloc (1, sizeof (gui_info_struct));
	/*
	 * Create the forms for guise 
	 */
	form = xvf_create_form(form_fullpath, NONE, run_guise, gui_info,
				clui_info->x_int, clui_info->y_int,
				XVF_PARTIAL_EDIT);
	if (form == NULL)
	   kexit(KEXIT_FAILURE);

	/*
	 * Initialize the GUI Info structure 
	 */
	if (!(_xvf_init_guise(form, gui_info)))
	    kexit(KEXIT_FAILURE);

/* -main_before_lib_call */
        guise_initialize();
/* -main_before_lib_call_end */

/* -main_library_call */
	xvf_run_form();
/* -main_library_call_end */

/* -main_after_lib_call */
	if (kfullpath(output_filename, kgetenv("TMPDIR"), fullpath) != NULL)
            kunlink(fullpath);

        kfree(output_filename);
/* -main_after_lib_call_end */

	kfree(form_filestring);
	kfree(form_pathname);
	kfree(form_fullpath);

	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: guise_usage_additions
| 
|       Purpose: Prints usage additions in guise_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname guise
|          Date: Apr 14, 1995
| Modifications: 
| 
------------------------------------------------------------*/
void guise_usage_additions(void)
{
	kfprintf(kstderr, "\tDirect manipulation GUI design tool\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: guise_free_args
| 
|       Purpose: Frees CLUI struct allocated in guise_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname guise
|          Date: Apr 14, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
guise_free_args(
    int   status,
    kaddr client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
		{
		kfree(clui_info->form_file);
	kfree(clui_info->o_file);
	kfree(clui_info->i_file);
		kfree(clui_info);
		}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
