#! /bin/sh

echo "running test1.sh for mextract_XXX ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb MATRIX -echo path`
dpath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}

# ------------------------------------------------------------------------
# Test  1: real, non-square, first row
# ------------------------------------------------------------------------
test_num=1
descr="real, non-square, first row"
cat <<EOF > $tmpdir/ascii.$$
2 3 4 1 
4 6 8 1
6 9 12 1
EOF
mextract_row -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 0
cat <<EOF > $tmpdir/ascii.$$
2 3 4 1 
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  2: real, non-square, last row
# ------------------------------------------------------------------------
test_num=2
descr="real, non-square, last row"
cat <<EOF > $tmpdir/ascii.$$
2 3 4 1
4 6 8 1
6 9 12 1
EOF
mextract_row -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 2
cat <<EOF > $tmpdir/ascii.$$
6 9 12 1
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  3: complex, non-square, first row
# ------------------------------------------------------------------------
test_num=3
descr="complex, non-square, first row"
cat <<EOF > $tmpdir/ascii.$$
(2,-5) (3,-5)
(4,-5) (6,-5)
(6,-5) (9,-5)
EOF
mextract_row -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 0
cat <<EOF > $tmpdir/ascii.$$
(2,-5) (3,-5)
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  4: complex, non-square, last row
# ------------------------------------------------------------------------
test_num=4
descr="complex, non-square, last row"
cat <<EOF > $tmpdir/ascii.$$
(2,-5) (3,-5)
(4,-5) (6,-5)
(6,-5) (9,-5)
EOF
mextract_row -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 2
cat <<EOF > $tmpdir/ascii.$$
(6,-5) (9,-5)
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  5: real, non-square, first col
# ------------------------------------------------------------------------
test_num=5
descr="real, non-square, first col"
cat <<EOF > $tmpdir/ascii.$$
2 3 4 1 
4 6 8 1
6 9 12 1
EOF
mextract_col -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 0
cat <<EOF > $tmpdir/ascii.$$
2
4
6
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  6: real, non-square, last col
# ------------------------------------------------------------------------
test_num=6
descr="real, non-square, last col"
cat <<EOF > $tmpdir/ascii.$$
2 3 4 1
4 6 8 1
6 9 12 1
EOF
mextract_col -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 3
cat <<EOF > $tmpdir/ascii.$$
1
1
1
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  7: complex, non-square, first col
# ------------------------------------------------------------------------
test_num=7
descr="complex, non-square, first col"
cat <<EOF > $tmpdir/ascii.$$
(2,-5) (3,-5)
(4,-5) (6,-5)
(6,-5) (9,-5)
EOF
mextract_col -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 0
cat <<EOF > $tmpdir/ascii.$$
(2,-5)
(4,-5)
(6,-5)
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  8: complex, non-square, last col
# ------------------------------------------------------------------------
test_num=8
descr="complex, non-square, last col"
cat <<EOF > $tmpdir/ascii.$$
(2,-5) (3,-5)
(4,-5) (6,-5)
(6,-5) (9,-5)
EOF
mextract_col -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -num 1
cat <<EOF > $tmpdir/ascii.$$
(3,-5)
(6,-5)
(9,-5)
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

exit $status

